#include <stdio.h>

#include "Polynomial.h"

//----------------------------
// read() - takes a degree,
//  rather than a length
//----------------------------

int Polynomial::read() {
  int degree;
  int rcode = scanf("%d", &degree);
  if ( rcode == EOF ) return EOF;
  if ( rcode != 1 ) throw "Error reading polynomial degree";
  if ( degree < 0 ) throw "Illegal polynomial degree";

  return fill(degree+1);
}

//-----------------------------
// print()
//-----------------------------

void Polynomial::print() {
  int n;

  for ( n=0; n<length && pVec[n]==0; n++ ); // skip initial elements with 0 coefficient
  if ( n <= length-1 ) {
    if ( n == 0 ) printf(" %lf", pVec[n]);
    else          printf(" %lfx**%d", pVec[n], n);
    
    for ( n=n+1; n<=length-1; n++) {
      if ( pVec[n] != 0.0 ) printf(" + %lfx**%d", pVec[n], n);
    }
  }
  else {
    printf("0");
  }
}

//------------------------------
// evaluate()
//------------------------------

double Polynomial::evaluate(double x) {
  int n;
  double xFactor = 1.0;
  double result = pVec[0];

  for ( n=1; n<length; n++ ) {
    xFactor *= x;
    result += pVec[n]*xFactor;
  }

  return result;
}
