#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "Vector.h"
#include "Utility.h"

//--------------------------
// the constructor
//--------------------------
Vector::Vector() {
  length = 0;
  capacity = 0;
  pVec = NULL;
}

//--------------------------
// the destructor - called
// when delete is used on 
// a Vector object
//--------------------------

Vector::~Vector() {
  if ( pVec == NULL ) return;
  free(pVec);
}

//--------------------------
// append() - protected
//   Add element to end.
//   We double capacity each
//   time that's necessary.
//--------------------------

void Vector::append(double x) {
  double* pNew;
  int     newCapacity;

  // an initial capacity of 1 is probably too small...
  if ( length == 0 ) newCapacity = 1;
  else newCapacity = capacity * 2;

  pNew = (double*)malloc( newCapacity * sizeof(double) );
  if ( !pNew ) throw "Out of memory";
  if ( capacity > 0 ) {
    bcopy(pVec, pNew, length*sizeof(double));
    free(pVec);
  }
  capacity = newCapacity;
  pVec = pNew;

  pVec[length++] = x;
}

//--------------------------
// fill() - reads data
//   elements from stdin
//--------------------------

int Vector::fill(int n) {
  for (int i=0; i<n; i++) {
    double x;
    if ( readDouble(x) == EOF ) return EOF;
    append(x);
  }

  return 0;
}


//--------------------------
// read from stdin
//   Return 0 for success.
//   Return non-zero for EOF
//--------------------------
int Vector::read() {
  int nElements;
  int rcode = scanf("%d", &nElements);
  if ( rcode == EOF ) return EOF;
  if ( rcode != 1 ) throw "Error reading vector length";
  if ( nElements < 0 ) throw "Illegal vector length";

  return fill(nElements);
}

//--------------------------
// print to stdout
//--------------------------
void Vector::print() {
  if ( length <= 0  ) return;
  for ( int i=0; i<length; i++) {
    printf(" %lf", pVec[i] );
  }
}
